// icon3dmorph - application object
//
// Copyright © 1999 Arcane Dragon Software
//					(arcanedragon@pobox.com)
//
// $Id$

#include "App.h"

#include <storage/NodeInfo.h>
#include <storage/Path.h>
#include <interface/Bitmap.h>
#include <interface/InterfaceDefs.h>
#include <support/String.h>
#include <stdio.h>
#include <errno.h>

// ----------------------------------------------------------------------
// Constructor/destructor

I3DApp::I3DApp( void )
	: BApplication( "application/x-vnd.ads.icon3dmorph" )
{
}

I3DApp::~I3DApp( void )
{
}

// ----------------------------------------------------------------------
// BApplication methods

bool I3DApp::QuitRequested( void )
{
	return true;
}

void I3DApp::ArgvReceived( int32 argc, char **argv )
{
	for( int32 i = 1; i < argc; i++ ) {
		printf( "ArgvReceived() on %s\n", argv[i] );

		entry_ref ref;
		status_t retval = get_ref_for_path( argv[i], &ref );

		if( retval == B_OK ) {
			retval = Make3DMorph( ref );
		}
	}

	PostMessage( B_QUIT_REQUESTED );
}

void I3DApp::RefsReceived( BMessage *msg )
{
	printf( "RefsReceived():\n" );
	msg->PrintToStream();

	entry_ref ref;

	int32 idx = 0;
	while( msg->FindRef( "refs", idx++, &ref ) == B_OK ) {
		printf( "RefsReceived() on item %d\n", idx - 1 );

		status_t retval = Make3DMorph( ref );
		// we ignore retval for now
		retval = retval;
	}

	PostMessage( B_QUIT_REQUESTED );
}

// ----------------------------------------------------------------------
// Private methods

status_t I3DApp::Make3DMorph( entry_ref& ref )
{
	// Grab the icon.
	BBitmap icon( BRect( 0, 0, B_LARGE_ICON - 1, B_LARGE_ICON - 1 ), 0, B_CMAP8 );
	status_t retval = BNodeInfo::GetTrackerIcon( &ref, &icon );
	if( retval != B_OK ) return retval;

	printf( "Got the icon.\n" );

	// Grab the system colour map.
	const color_map *colours = system_colors();
	const rgb_color *colours_8bit = colours->color_list;

	// Now, where's a good spot to put the output?  In with the dropped
	// file!
	BPath path( &ref );
	if( path.InitCheck() != B_OK ) return path.InitCheck();
	
	BString str( path.Path() );
	str += ".3dmorph";

	printf( "Opening %s...\n", str.String() );

	FILE *fp = fopen( str.String(), "w" );
	if( fp == NULL ) return errno;

	// Write out the 3dmorph file.
	fprintf( fp, "; 3dmorph object, written by icon3dmorph\n" );
	fprintf( fp, "; Courtesy of Arcane Dragon Software (arcanedragon@pobox.com)\n" );

	// header
	fprintf( fp, "#N%d\n", B_LARGE_ICON * B_LARGE_ICON );

	// data
	uint8 *bits = static_cast<uint8 *>( icon.Bits() );

	printf( "bits = %08x, bpr = %d\n", bits, icon.BytesPerRow() );

	for( int y = 0; y < B_LARGE_ICON; y++ ) {
		for( int x = 0; x < B_LARGE_ICON; x++ ) {
			uint32 offset = y * icon.BytesPerRow() + x;

			float fx = (float)x / 31.0;
			float fy = (float)y / 31.0;
printf( "bits[%d, %d] = %d\n", x, y, bits[offset] );
			float fz = (float)(colours_8bit[bits[offset]].alpha) / 255.0;
			float fr = (float)(colours_8bit[bits[offset]].red) / 255.0;
			float fg = (float)(colours_8bit[bits[offset]].green) / 255.0;
			float fb = (float)(colours_8bit[bits[offset]].blue) / 255.0;

			printf( "(%f, %f, %f) = r: %f, g: %f, b: %f\n", fx, fy, fz, fr, fg, fb );
			fprintf( fp, "#P%f %f %f %f %f %f\n", fx, fy, fz, fr, fg, fb );
		}
	}

	// Done!
	fclose( fp );

	// Now give it a good file type.
	BNode node( str.String() );
	if( node.InitCheck() != B_OK ) return node.InitCheck();
	
	BNodeInfo ni( &node );
	if( ni.InitCheck() != B_OK ) return ni.InitCheck();
	
	return ni.SetType( "text/plain" );
}
